package utils

import (
	"encoding/json"
	"github.com/beego/beego/v2/adapter/logs"
	"os"
	"path"
)

const (
	DefaultLogPath    = "./logs/filelog"
	DefaultBinlogPath = "./logs/binlog"
	DefaultDataPath   = "./logs/datalog"
	DefaultApiLogPath = "./logs/apilog"
)

// 日志配置
var (
	LogPath     string //调用过程中的日志存放地址
	LogFile     string
	LogDataPath string //调用过程中图表相关的日志存放地址
	LogDataFile string
	BinLogPath  string //数据库相关的日志存放地址
	BinLogFile  string
	ApiLogPath  string //接口请求地址和接口返回值日志存放地址
	ApiLogFile  string
	LogMaxDays  int //日志最大保留天数
)

var FileLog *logs.BeeLogger
var ApiLog *logs.BeeLogger
var FileLogData *logs.BeeLogger
var Binlog *logs.BeeLogger

func init() {
	if LogMaxDays == 0 {
		LogMaxDays = 31
	}
	logPath := LogPath
	if logPath == "" {
		logPath = DefaultLogPath
	}
	logFile := LogFile
	if logFile == "" {
		logFile = "filelog.log"
	}
	os.MkdirAll(logPath, os.ModePerm)

	// 打开文件
	logFileName := path.Join(logPath, logFile)
	FileLog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	b, _ := json.Marshal(logConf)
	FileLog.SetLogger(logs.AdapterFile, string(b))
	FileLog.EnableFuncCallDepth(true)

	initBinlog()
	initApiLog()
	initFileLogData()
}

type logConfig struct {
	FileName string `json:"filename" description:"保存的文件名"`
	MaxLines int    `json:"maxlines"  description:"每个文件保存的最大行数，默认值 1000000"`
	MaxSize  int    `json:"maxsize" description:"每个文件保存的最大尺寸，默认值是 1 << 28, //256 MB"`
	Daily    bool   `json:"daily" description:"是否按照每天 logrotate，默认是 true"`
	MaxDays  int    `json:"maxdays" description:"文件最多保存多少天，默认保存 7 天"`
	Rotate   bool   `json:"rotate" description:"是否开启 logrotate，默认是 true"`
	Level    int    `json:"level" description:"日志保存的时候的级别，默认是 Trace 级别"`
	Color    bool   `json:"color" description:"日志是否输出颜色"`
	//Perm     string `json:"perm" description:"日志文件权限"`
}

func initBinlog() {
	binlogPath := BinLogPath
	if binlogPath == "" {
		binlogPath = DefaultBinlogPath
	}
	binlogFile := BinLogFile
	if binlogFile == "" {
		binlogFile = "binlog.log"
	}
	os.MkdirAll(binlogPath, os.ModePerm)
	logFileName := path.Join(binlogPath, binlogFile)
	Binlog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	b, _ := json.Marshal(logConf)
	Binlog.SetLogger(logs.AdapterFile, string(b))
	Binlog.EnableFuncCallDepth(true)
}

func initApiLog() {
	logPath := ApiLogPath
	if logPath == "" {
		logPath = DefaultApiLogPath
	}
	logFile := ApiLogFile
	if logFile == "" {
		logFile = "apilog.log"
	}
	os.MkdirAll(logPath, os.ModePerm)

	// 打开文件
	logFileName := path.Join(logPath, logFile)
	ApiLog = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	b, _ := json.Marshal(logConf)
	ApiLog.SetLogger(logs.AdapterFile, string(b))
	ApiLog.EnableFuncCallDepth(true)
}

func initFileLogData() {
	logPath := LogDataPath
	if logPath == "" {
		logPath = DefaultDataPath
	}
	logFile := LogDataFile
	if logFile == "" {
		logFile = "datalog.log"
	}
	os.MkdirAll(logPath, os.ModePerm)

	// 打开文件
	logFileName := path.Join(logPath, logFile)
	FileLogData = logs.NewLogger(1000000)
	logConf := getDefaultLogConfig()

	logConf.FileName = logFileName
	b, _ := json.Marshal(logConf)
	FileLogData.SetLogger(logs.AdapterFile, string(b))
	FileLogData.EnableFuncCallDepth(true)
}

func getDefaultLogConfig() logConfig {
	return logConfig{
		FileName: "",
		MaxLines: 10000000,
		MaxSize:  1 << 28,
		Daily:    true,
		MaxDays:  LogMaxDays, //我就是喜欢31天，咋滴，不喜欢你就自己改-_-!
		Rotate:   true,
		Level:    logs.LevelTrace,
		//Perm:     "",
	}
}
